#include "CChargedParticle.h"

#include <fstream>

using namespace std;
using namespace NApi;
using namespace NApiCore;
using namespace NApiPbf;

const string CChargedParticle::PREFS_FILE = "charged_particle_prefs.txt";
const double CChargedParticle::EPSILON0   = 8.854e-12;

CChargedParticle::CChargedParticle()
{
    ;
}

CChargedParticle::~CChargedParticle()
{
    ;
}

void CChargedParticle::getPreferenceFileName(char prefFileName[FILE_PATH_MAX_LENGTH])
{
    // Copy in pref file name
    strcpy(prefFileName, PREFS_FILE.c_str());
}

bool CChargedParticle::isThreadSafe()
{
    // thread safe
    return true;
}

bool CChargedParticle::setup(NApiCore::IApiManager_1_0& apiManager,
                                 const char                 prefFile[])
{
    ifstream prefsFile(prefFile);

    if(!prefsFile)
    {
        return false;
    }
    else
    {
        double pointX, pointY, pointZ;
        prefsFile >> pointX >> pointY >> pointZ;
        m_plates = CSimple3DVector(pointX,pointY,pointZ);

        prefsFile >> m_freq >> m_point >> m_particles;

        prefsFile >> pointX >> pointY >> pointZ;
        m_pointPos = CSimple3DPoint(pointX,pointY,pointZ);

        prefsFile >> pointX >> pointY >> pointZ;
        m_platesLower = CSimple3DPoint(pointX,pointY,pointZ);

        prefsFile >> pointX >> pointY >> pointZ;
        m_platesUpper = CSimple3DPoint(pointX,pointY,pointZ);
    }

    return true;
}

ECalculateResult CChargedParticle::externalForce(
                                                 double       time,
                                                 double       timestep,
                                                 int          id,
                                                 const char   type[],
                                                 double       mass,
                                                 double       volume,
                                                 double       posX,
                                                 double       posY,
                                                 double       posZ,
                                                 double       velX,
                                                 double       velY,
                                                 double       velZ,
                                                 double       angVelX,
                                                 double       angVelY,
                                                 double       angVelZ,
                                                 double       charge,
                                                 const double orientation[9],
                                                 NApiCore::ICustomPropertyDataApi_1_0* particlePropData,
                                                 NApiCore::ICustomPropertyDataApi_1_0* simulationPropData,
                                                 double&      calculatedForceX,
                                                 double&      calculatedForceY,
                                                 double&      calculatedForceZ,
                                                 double&      calculatedTorqueX,
                                                 double&      calculatedTorqueY,
                                                 double&      calculatedTorqueZ)
{
    // This simulation consists of two plates parallel to the primary
    // axes providing a time varying sinusoidal electrostatic field
    // strength m_plates. Particles are deflected through the plates
    // There is also a point source past the plates with a charge qPlate
    // particles have charge of qParticle
    CSimple3DPoint  nPos(posX, posY, posZ);
    CSimple3DVector F_plate, F_point;

    // Plate force
    if (nPos.x() > m_platesLower.x() && nPos.x() < m_platesUpper.x() &&
        nPos.y() > m_platesLower.y() && nPos.y() < m_platesUpper.y() &&
        nPos.z() > m_platesLower.z() && nPos.z() < m_platesUpper.z())
    {
        F_plate = m_plates * m_particles * cos(m_freq * time);
    }

    //point force: F = m_point * m_particles / (4 * PI * E_0 * r^2) * vector joining the charges
    CSimple3DVector contactVector = nPos - m_pointPos;
    double multipleFactor = 1.0/ (4 * PI * EPSILON0 * contactVector.lengthSquared());
    contactVector.normalise();

    F_point = contactVector * m_point * m_particles * multipleFactor;

    CSimple3DVector F_total = F_plate + F_point;
    calculatedForceX = F_total.dx();
    calculatedForceY = F_total.dy();
    calculatedForceZ = F_total.dz();

    return eSuccess;
}