#include "CCohesion.h"

#include "Helpers.h"

using namespace std;
using namespace NApi;
using namespace NApiCore;
using namespace NApiCm;

const string CCohesion::PREFS_FILE = "cohesion_prefs.txt";

CCohesion::CCohesion() :
                 m_cohesiveItems()
{
    ;
}

CCohesion::~CCohesion()
{
    ;
}

void CCohesion::getPreferenceFileName(char prefFileName[FILE_PATH_MAX_LENGTH])
{
    // Copy in pref file name
    strcpy(prefFileName, PREFS_FILE.c_str());
}

bool CCohesion::setup(NApiCore::IApiManager_1_0& apiManager,
                      const char                 prefFile[])
{
    ifstream prefsFile(prefFile);

    if(!prefsFile)
    {
        return false;
    }
    else
    {
        double energyDensity;
        string str;
        while (prefsFile)
        {
            prefsFile >> str
                      >> energyDensity;

            string::size_type i (str.find (':'));
            string surfA = str.substr (0, i);
            str.erase (0, i + 1);
            string surfB = str;

            m_cohesiveItems.addCohesion(surfA, surfB,energyDensity);
        }
    }

    return true;
}

ECalculateResult CCohesion::calculateForce(double       time,
                                           double       timestep,
                                           int          elem1Id,
                                           const char   elem1Type[],
                                           double       elem1Mass,
                                           double       elem1ShearMod,
                                           double       elem1Poisson,
                                           double       elem1ContactCurvature,
                                           double       elem1PhysicalCurvature,
                                           double       elem1PosX,
                                           double       elem1PosY,
                                           double       elem1PosZ,
                                           double       elem1ComX,
                                           double       elem1ComY,
                                           double       elem1ComZ,
                                           double       elem1VelX,
                                           double       elem1VelY,
                                           double       elem1VelZ,
                                           double       elem1AngVelX,
                                           double       elem1AngVelY,
                                           double       elem1AngVelZ,
                                           double       elem1Charge,
                                           double       elem1WorkFunction,
                                           const double elem1Orientation[9],
                                           NApiCore::ICustomPropertyDataApi_1_0* elem1PropData,
                                           bool         elem2IsSurf,
                                           int          elem2Id,
                                           const char   elem2Type[],
                                           double       elem2Mass,
                                           double       elem2Area,
                                           double       elem2ShearMod,
                                           double       elem2Poisson,
                                           double       elem2ContactCurvature,
                                           double       elem2PhysicalCurvature,
                                           double       elem2PosX,
                                           double       elem2PosY,
                                           double       elem2PosZ,
                                           double       elem2ComX,
                                           double       elem2ComY,
                                           double       elem2ComZ,
                                           double       elem2VelX,
                                           double       elem2VelY,
                                           double       elem2VelZ,
                                           double       elem2AngVelX,
                                           double       elem2AngVelY,
                                           double       elem2AngVelZ,
                                           double       elem2Charge,
                                           double       elem2WorkFunction,
                                           const double elem2Orientation[9],
                                           NApiCore::ICustomPropertyDataApi_1_0* elem2PropData,
                                           NApiCore::ICustomPropertyDataApi_1_0* contactPropData,
                                           NApiCore::ICustomPropertyDataApi_1_0* simulationPropData,
                                           double       coeffRest,
                                           double       staticFriction,
                                           double       rollingFriction,
                                           double       contactPointX,
                                           double       contactPointY,
                                           double       contactPointZ,
                                           double       normalOverlap,
                                           double&      tangentialOverlapX,
                                           double&      tangentialOverlapY,
                                           double&      tangentialOverlapZ,
                                           double&      calculatedNormalForceX,
                                           double&      calculatedNormalForceY,
                                           double&      calculatedNormalForceZ,
                                           double&      calculatedUnsymNormalForceX,
                                           double&      calculatedUnsymNormalForceY,
                                           double&      calculatedUnsymNormalForceZ,
                                           double&      calculatedTangentialForceX,
                                           double&      calculatedTangentialForceY,
                                           double&      calculatedTangentialForceZ,
                                           double&      calculatedUnsymTangentialForceX,
                                           double&      calculatedUnsymTangentialForceY,
                                           double&      calculatedUnsymTangentialForceZ,
                                           double&      calculatedElem1AdditionalTorqueX,
                                           double&      calculatedElem1AdditionalTorqueY,
                                           double&      calculatedElem1AdditionalTorqueZ,
                                           double&      calculatedElem1UnsymAdditionalTorqueX,
                                           double&      calculatedElem1UnsymAdditionalTorqueY,
                                           double&      calculatedElem1UnsymAdditionalTorqueZ,
                                           double&      calculatedElem2AdditionalTorqueX,
                                           double&      calculatedElem2AdditionalTorqueY,
                                           double&      calculatedElem2AdditionalTorqueZ,
                                           double&      calculatedElem2UnsymAdditionalTorqueX,
                                           double&      calculatedElem2UnsymAdditionalTorqueY,
                                           double&      calculatedElem2UnsymAdditionalTorqueZ,
                                           double&      calculatedChargeMovedToElem1)
{
    // This contact model assumes that the Physical and the Contact radius
    // are the same

    // The unit vector from element 1 to the contact point
    CSimple3DPoint  contactPoint = CSimple3DPoint(contactPointX, contactPointY, contactPointZ);
    CSimple3DVector unitCPVect   = contactPoint - CSimple3DPoint(elem1PosX, elem1PosY, elem1PosZ);
    unitCPVect.normalise();

    // Cohesion
    // R^2 - (R - d)^2

    double nCohesiveFactor         = m_cohesiveItems.getCohesion(elem1Type, elem2Type);
    double nRadiusOfOverlapSquared = 2 * elem1PhysicalCurvature * normalOverlap;
    double nArea                   = PI * nRadiusOfOverlapSquared;
    CSimple3DVector F_cohesive     = unitCPVect * nCohesiveFactor * nArea;

    calculatedNormalForceX = F_cohesive.dx();
    calculatedNormalForceY = F_cohesive.dy();
    calculatedNormalForceZ = F_cohesive.dz();

    return eSuccess;
}