#if !defined(CFluidDrag_h)
#define CFluidDrag_h

#include <string>

#include "Helpers.h"

#include "IPluginParticleBodyForceV2_1_0.h"

/**
 * This class provides an implementation of IPluginParticleBodyForceV2_1_0
 * That applies an fluid drag force to particles.
 *
 * This plugin reads a config file from the same directory as
 * it is stored in.  The file format is as follows (1 line per value):
 *     inlet_x
 *     inlet_y
 *     inlet_z
 *     outlet_x
 *     outlet_y
 *     outlet_z
 *     diameter
 *     fluid_velocity
 *     drag_coefficient
 *     density
 *
 * NOTE: This version of the fluid drage code is provided only
 * as an example of how to create a particle body force
 * plugin and is not updated on a regular basis.
 */
class CFluidDrag : public NApiPbf::IPluginParticleBodyForceV2_1_0
{
    public:
        /**
         * Name of the preferences file to load bond information
         * from
         */
        static const std::string PREFS_FILE;

        /**
         * Constructor, does nothing
         */
        CFluidDrag();

        /**
         * Destructor, does nothing
         */
        virtual ~CFluidDrag();

        /**
         * Sets prefFileName to PREFS_FILE
         *
         *
         * Implementation of method from IPluginParticleBodyForceV2_1_0
         */
        virtual void getPreferenceFileName(char prefFileName[NApi::FILE_PATH_MAX_LENGTH]);

        /**
         * Returns true to indicate plugin is thread safe
         *
         * Implementation of method from IPluginParticleBodyForceV2_1_0
         */
        virtual bool isThreadSafe();

        /**
         * Reads config file
         *
         * Implementation of method from IPluginParticleBodyForceV2_1_0
         */
        virtual bool setup(NApiCore::IApiManager_1_0& apiManager,
                           const char                 prefFile[]);

        /**
         * Applies an fluid drag force to particles
         *
         * Implementation of method from IPluginParticleBodyForceV2_1_0
         */
        virtual NApi::ECalculateResult externalForce(
                                               double       time,
                                               double       timestep,
                                               int          id,
                                               const char   type[],
                                               double       mass,
                                               double       volume,
                                               double       posX,
                                               double       posY,
                                               double       posZ,
                                               double       velX,
                                               double       velY,
                                               double       velZ,
                                               double       angVelX,
                                               double       angVelY,
                                               double       angVelZ,
                                               double       charge,
                                               const double orientation[9],
                                               NApiCore::ICustomPropertyDataApi_1_0* particlePropData,
                                               NApiCore::ICustomPropertyDataApi_1_0* simulationPropData,
                                               double&      calculatedForceX,
                                               double&      calculatedForceY,
                                               double&      calculatedForceZ,
                                               double&      calculatedTorqueX,
                                               double&      calculatedTorqueY,
                                               double&      calculatedTorqueZ);

    private:
        CSimple3DPoint m_inlet;
        CSimple3DPoint m_outlet;
        double         m_diameter;
        double         m_fluidVeloctiy;
        double         m_dragCoefficient;
        double         m_density;

};

#endif
