#if !defined(cresidencetime_h)
#define cresidencetime_h

#include <string>

#include "IPluginParticleBodyForceV2_1_0.h"

/**
 * This class provides an implementation of IPluginParticleBodyForceV2_1_0
 * That adds and updates a residence time property to each particle
 * in the system.
 */
class CResidenceTime : public NApiPbf::IPluginParticleBodyForceV2_1_0
{
    public:
        /**
         * Name of the residence time custom property
         */
        static const std::string RESIDENCE_TIME_PROPERTY;

        /**
         * Constructor, does nothing
         */
        CResidenceTime();

        /**
         * Destructor, does nothing
         */
        virtual ~CResidenceTime();

        /**
         * Returns true to indicate plugin is thread safe
         *
         * Implementation of method from IPluginParticleBodyForceV2_1_0
         */
        virtual bool isThreadSafe();

        /**
         * Returns true to indicate the plugin uses custom properties
         *
         * Implementation of method from IPluginParticleBodyForceV2_1_0
         */
        virtual bool usesCustomProperties();

        /**
         *
         *
         * Implementation of method from IPluginParticleBodyForceV2_1_0
         */
        virtual NApi::ECalculateResult externalForce(
                                               double       time,
                                               double       timestep,
                                               int          id,
                                               const char   type[],
                                               double       mass,
                                               double       volume,
                                               double       posX,
                                               double       posY,
                                               double       posZ,
                                               double       velX,
                                               double       velY,
                                               double       velZ,
                                               double       angVelX,
                                               double       angVelY,
                                               double       angVelZ,
                                               double       charge,
                                               const double orientation[9],
                                               NApiCore::ICustomPropertyDataApi_1_0* particlePropData,
                                               NApiCore::ICustomPropertyDataApi_1_0* simulationPropData,
                                               double&      calculatedForceX,
                                               double&      calculatedForceY,
                                               double&      calculatedForceZ,
                                               double&      calculatedTorqueX,
                                               double&      calculatedTorqueY,
                                               double&      calculatedTorqueZ);

        /**
         * Returns 1 to indicate the plugin wishes to register 1 property
         *
         * Implementation of method from IPluginParticleBodyForceV2_1_0
         */
        virtual unsigned int getNumberOfRequiredProperties(
                                 const NApi::EPluginPropertyCategory category);

        /**
         * Returns details for the residence time property
         *
         * Implementation of method from IPluginParticleBodyForceV2_1_0
         */
        virtual bool getDetailsForProperty(
                             unsigned int                    propertyIndex,
                             NApi::EPluginPropertyCategory   category,
                             char                            name[NApi::CUSTOM_PROP_MAX_NAME_LENGTH],
                             NApi::EPluginPropertyDataTypes& dataType,
                             unsigned int&                   numberOfElements,
                             NApi::EPluginPropertyUnitTypes& unitType);
};

#endif
