#include "CSnooker.h"

#include <stdio.h>
#include <string.h>

using namespace NApi;
using namespace NApiCore;
using namespace NApiFactory;

CSnooker::CSnooker() :
          m_numberCreated(0)
{

}

CSnooker::~CSnooker()
{

}

NApi::ECalculateResult CSnooker::createParticle(
                                   double  time,
                                   bool&   particleCreated,
                                   bool&   additionalParticleRequired,
                                   char    type[API_BASIC_STRING_LENGTH],
                                   double& scale,
                                   double& posX,
                                   double& posY,
                                   double& posZ,
                                   double& velX,
                                   double& velY,
                                   double& velZ,
                                   double& angVelX,
                                   double& angVelY,
                                   double& angVelZ,
                                   double  orientation[9],
                                   ICustomPropertyDataApi_1_0* propData)
{

    if(time > 1e-7 || m_numberCreated >=22)
    {
        particleCreated = false;
        additionalParticleRequired = false;
        return eSuccess;
    }

    // Mark that we are creating a particle
    particleCreated = true;

    // Set some fixed values for all particles we create
    velX = 0.0;
    velY = 0.0;
    velZ = 0.0;

    angVelX = 0.0;
    angVelY = 0.0;
    angVelZ = 0.0;

    for (unsigned int i = 0; i < 9; i++)
    {
        orientation[i] = 0.0;
    }

    scale = 1.0;

    // Z position is the same for all aprticles (just above the table
    // X and Y vary
    posZ = 0.025;

    // Set particle specific parameters (location)
    switch(m_numberCreated)
    {
        //reds
        default:
            strcpy(type,"red");
            if(m_numberCreated <= 4)
            {
                posY = 0.55 - 0.05*(4-m_numberCreated);
                posX = 0.324;
            }
            if(m_numberCreated > 4 && m_numberCreated <=8)
            {
                posY = 0.525 - 0.05*(8-m_numberCreated);
                posX = 0.368;
            }
            if(m_numberCreated > 8 && m_numberCreated <= 11)
            {
                posY = 0.5 - 0.05*(11-m_numberCreated);
                posX = 0.412;
            }
            if(m_numberCreated > 11 && m_numberCreated <= 13)
            {
                posY = 0.475 - 0.05*(13-m_numberCreated);
                posX = 0.456;
            }
            if(m_numberCreated == 14)
            {
                posY = 0.45;
                posX = 0.5;
            }
            break;

         //black
        case(15):
            strcpy(type,"black");
            posY = 0.45;
            posX = 0.15;
            break;

        //pink
        case(16):
            strcpy(type,"pink");
            posY = 0.45;
            posX = 0.55;
            break;

        //blue
        case(17):
            strcpy(type,"blue");
            posY = 0.45;
            posX = 0.9;
            break;

        //brown
        case(18):
            strcpy(type,"brown");
            posY = 0.45;
            posX = 1.4;
            break;

        //green
        case(19):
            strcpy(type,"green");
            posY = 0.25;
            posX = 1.4;
            break;

        //yellow
        case(20):
            strcpy(type,"yellow");
            posY = 0.65;
            posX = 1.4;
            break;

        //white
        case(21):
            strcpy(type,"white");
            posY = 0.3;
            posX = 1.6;

            // Override the starting velocity
            // We only start with the white ball moving
            velX = -2.5;
            velY = 0.2;
            break;
    }

    // Mark the particle as created
    m_numberCreated++;

    // We are creating 21 particles, if we've done them all
    // then no more are required
    additionalParticleRequired = (22 != m_numberCreated);

    return eSuccess;
}